@@ -57,22 +57,50 @@
 // ----- local constants
 
 // ----- local types ----------
+typedef struct tagTimings_t
+{
+  REAL8 Total;		// total time spent in XLALComputeFstatResamp()
+  REAL8 Bary;		// time spent in barycentric resampling
+  REAL8 Spin;		// time spent in spindown+frequency correction
+  REAL8 FFT;		// time spent in FFT
+  REAL8 Copy;		// time spent copying results from FFT to FabX
+  REAL8 Norm;		// time spent normalizing the final Fa,Fb
+  REAL8 Fab2F;		// time to compute Fstat from {Fa,Fb}
+  REAL8 Mem;		// time to realloc and Memset-0 arrays
+  REAL8 SumFabX;	// time to sum_X Fab^X
+} Timings_t;
 
-// ----- workspace ----------
 typedef struct tagResampTimingInfo
 { // NOTE: all times refer to a single-detector timing case
-  REAL8 tauTotal;		// total time spent in XLALComputeFstatResamp()
-  REAL8 tauBary;		// time spent in barycentric resampling
-  REAL8 tauSpin;		// time spent in spindown+frequency correction
-  REAL8 tauFFT;			// time spent in FFT
-  REAL8 tauNorm;		// time spent normalizing the final Fa,Fb
-  REAL8 tauFab2F;		// time to compute Fstat from {Fa,Fb}
-  REAL8 tauMem;			// time to realloc and memset-0 arrays
-  REAL8 tauSumFabX;		// time to sum_X Fab^X
-  REAL8 tauF1Buf;		// Resampling timing 'constant': Fstat time per template per detector for a 'buffered' case (same skypos, same numFreqBins)
-  REAL8 tauF1NoBuf;		// Resampling timing 'constant': Fstat time per template per detector for an 'unbuffered' usage (different skypos and numFreqBins)
+  BOOLEAN collectTiming;	// turn on/off the collection of F-stat-method-specific timing-data (stored in workspace)
+
+  UINT4 NFbin;		// number of frequency bins to compute F-stat for
+  UINT4 Nsft;		// total number of SFTs used
+  UINT4 Ndet;		// number of detectors
+  REAL8 Resolution;	// frequency resolution in 'natural' units of 1/Tspan: R = T_span/T_FFT in [0, 1]
+  UINT4 NsFFT0;		// 'original' length of barycentered timeseries to be FFT'ed
+  UINT4 NsFFT;		// actual length of FFT, potentially rounded up to power-of-2 for efficiency
+
+  Timings_t Tau;
+
+  // ------------------------------------------------------------
+  // Resampling timing model in terms of 'fundamental' coefficients tau_Fbin, tau_spin, tau_FFT:
+  // ==> tau_RS = tau_Fbin + (NsFFT / NFbin) * [ R * tau_spin + tau_FFT ]
+  // The total runtime per call (over NFbin for one detector) will generally have an additional contribution from barycentering:
+  // ==> Tau.Total = NFbin * tau_RS + b * R * NsFFT * tau_bary,
+  // where b = 1/N_{f1dot,f2dot,...} is the buffering weight applied to the barycentering contribution, which generally
+  // will make b<<1 if there's many spindown template per sky- and binary-orbital templates.
+  REAL8 tau_RS;		// measured resampling F-stat time per output F-stat bin (excluding barycentering): = (Tau.Total - Tau.Bary) / NFbin
+  REAL8 tau_Fbin;	// time contribution of operations that scale exactly with numFreqBins, expressible as
+                        // tau_Fbin = (Tau.Copy + Tau.Norm + Tau.SumFabX + Tau.Fab2F) / NFbin
+  REAL8 tau_FFT;	// FFT time per FFT sample: tau_FFT = Tau.FFT / NsFFT
+  REAL8 tau_spin;	// time contribution from applying spindown-correction (and freq-shift), per SRC-frame sample: tau_spin = Tau.Spin /(R* NsFFT)
+  REAL8 tau_bary;	// barycentering time per SRC-frame sample, assuming no buffering: tau_bary = Tau.Bary /(R * NsFFT)
+  // ------------------------------------------------------------
+
 } ResampTimingInfo;
 
+// ----- workspace ----------
 typedef struct tagResampWorkspace
 {
   // intermediate quantities to interpolate and operate on SRC-frame timeseries
@@ -95,7 +123,7 @@ typedef struct tagResampWorkspace
   COMPLEX8 *Fb_k;		// properly normalized F_b(f_k) over output bins
   UINT4 numFreqBinsAlloc;	// internal: keep track of allocated length of frequency-arrays
 
-  ResampTimingInfo timingInfo;	// temporary storage for collecting timing data
+  ResampTimingInfo *timingInfo;	// pointer to storage for collecting timing data (which lives in ResampMethodData)
 } ResampWorkspace;
 
 typedef struct
@@ -110,7 +138,7 @@ typedef struct
   MultiCOMPLEX8TimeSeries *multiTimeSeries_SRC_a;	// multi-detector SRC-frame timeseries, multiplied by AM function a(t)
   MultiCOMPLEX8TimeSeries *multiTimeSeries_SRC_b;	// multi-detector SRC-frame timeseries, multiplied by AM function b(t)
 
-  FILE *timingLogFile;					// file to write timing info to
+  ResampTimingInfo timingInfo;				// temporary storage for collecting timing data
 } ResampMethodData;
 
 
@@ -145,6 +173,11 @@ XLALComputeFaFb_Resamp ( ResampWorkspace *ws,
                          const COMPLEX8TimeSeries *TimeSeries_SRC_b
                          );
 
+static void
+XLALGetFFTPlanHints ( int * planMode,
+                      double * planGenTimeoutSeconds
+                      );
+
 // ==================== function definitions ====================
 
 static void
@@ -173,36 +206,6 @@ XLALDestroyResampWorkspace ( void *workspace )
 
 } // XLALDestroyResampWorkspace()
 
-/// debug/optimizer helper function: dump internal info from resampling code into a file
-static void
-AppendResampInfo2File ( FILE *fp, const FstatCommon *common, const ResampMethodData *resamp )
-{
-
-  // print header on first call
-  static BOOLEAN print_header = 1;
-  if ( print_header ) {
-    fprintf (fp, "%%%%%8s %10s %6s %10s %10s ",
-             "Nfreq", "NsFFT", "Nsft0", "Ns_DET0", "Ns_SRC0" );
-    fprintf (fp, "%10s %10s %10s %10s %10s %10s %10s %10s %10s %10s %10s\n",
-             "tauTotal", "tauFFT", "tauBary", "tauSpin", "tauAM", "tauNorm", "tauFab2F", "tauMem", "tauSumFabX", "tauF1NoBuf", "tauF1Buf" );
-    print_header = 0;
-  }
-
-  const ResampWorkspace *ws = (ResampWorkspace*) common->workspace;
-
-  fprintf (fp, "%10d %10d", ws->numFreqBinsOut, ws->numSamplesFFT );
-  UINT4 numSamples_DETX0 = resamp->multiTimeSeries_DET->data[0]->data->length;
-  UINT4 numSFTs_X0 = common->multiTimestamps->data[0]->length;
-  COMPLEX8TimeSeries *ts_SRCX0 = resamp->multiTimeSeries_SRC_a->data[0];
-  UINT4 numSamples_SRCX0 = ts_SRCX0->data->length;
-  fprintf (fp, " %6d %10d %10d ", numSFTs_X0, numSamples_DETX0, numSamples_SRCX0 );
-
-  const ResampTimingInfo *ti = &(ws->timingInfo);
-  fprintf (fp, "%10.1e %10.1e %10.1e %10.1e %10.1e %10.1e %10.1e %10.1e %10.1e %10.1e %10.1e\n",
-           ti->tauTotal, ti->tauFFT, ti->tauBary, ti->tauSpin, 0.0, ti->tauNorm, ti->tauFab2F, ti->tauMem, ti->tauSumFabX, ti->tauF1NoBuf, ti->tauF1Buf );
-
-} // AppendResampInfo2File()
-
 // ---------- internal functions ----------
 static void
 XLALDestroyResampMethodData ( void* method_data )
@@ -271,12 +274,11 @@ XLALSetupFstatResamp ( void **method_data,
   }
   TspanFFT *= decimateFFT;
 
-  UINT4 numSamplesFFT = (UINT4) ceil ( TspanFFT / dt_DET );      // we use ceil() so that we artificially widen the band rather than reduce it
-  // round numSamplesFFT to next power of 2
-  numSamplesFFT = (UINT4) pow ( 2, ceil(log2(numSamplesFFT)));
+  UINT4 numSamplesFFT0 = (UINT4) ceil ( TspanFFT / dt_DET );      // we use ceil() so that we artificially widen the band rather than reduce it
+  UINT4 numSamplesFFT = (UINT4) pow ( 2, ceil ( log2 ( numSamplesFFT0 ) ) );  // round numSamplesFFT up to next power of 2 for most effiecient FFT
   REAL8 dt_SRC = TspanFFT / numSamplesFFT;			// adjust sampling rate to allow achieving exact requested dFreq=1/TspanFFT !
 
-  // ----- allocate buffer memory ----------
+  // ----- allocate buffer Memory ----------
 
   // header for SRC-frame resampled timeseries buffer
   XLAL_CHECK ( (resamp->multiTimeSeries_SRC_a = XLALCalloc ( 1, sizeof(MultiCOMPLEX8TimeSeries)) ) != NULL, XLAL_ENOMEM );
@@ -301,7 +303,7 @@ XLALSetupFstatResamp ( void **method_data,
       REAL8 TsftX = common->multiTimestamps->data[X]->deltaT;
       XLAL_CHECK ( Tsft == TsftX, XLAL_EINVAL, "Input timestamps must have identical stepsize 'Tsft(X=%d)' (%.16g != %.16g)\n", X, Tsft, TsftX );
 
-      // ----- prepare memory fo SRC-frame timeseries and AM coefficients
+      // ----- prepare Memory fo SRC-frame timeseries and AM coefficients
       const char *nameX = resamp->multiTimeSeries_DET->data[X]->name;
       UINT4 numSamples_DETX = resamp->multiTimeSeries_DET->data[X]->data->length;
       UINT4 numSamples_SRCX = (UINT4)ceil ( numSamples_DETX * dt_DET / dt_SRC );
@@ -320,14 +322,21 @@ XLALSetupFstatResamp ( void **method_data,
     {
       if ( numSamplesFFT > ws->numSamplesFFT )
         {
+          int fft_plan_flags=FFTW_MEASURE;
+          double fft_plan_timeout= FFTW_NO_TIMELIMIT ;
+
           fftw_free ( ws->FabX_Raw );
           XLAL_CHECK ( (ws->FabX_Raw = fftw_malloc ( numSamplesFFT * sizeof(COMPLEX8) )) != NULL, XLAL_ENOMEM );
           fftw_free ( ws->TS_FFT );
           XLAL_CHECK ( (ws->TS_FFT   = fftw_malloc ( numSamplesFFT * sizeof(COMPLEX8) )) != NULL, XLAL_ENOMEM );
 
           LAL_FFTW_WISDOM_LOCK;
+          XLALGetFFTPlanHints (& fft_plan_flags , & fft_plan_timeout);
+
           fftwf_destroy_plan ( ws->fftplan );
-          XLAL_CHECK ( (ws->fftplan = fftwf_plan_dft_1d ( numSamplesFFT, ws->TS_FFT, ws->FabX_Raw, FFTW_FORWARD, FFTW_MEASURE )) != NULL, XLAL_EFAILED, "fftwf_plan_dft_1d() failed\n");
+
+          fftw_set_timelimit( fft_plan_timeout );
+          XLAL_CHECK ( (ws->fftplan = fftwf_plan_dft_1d ( numSamplesFFT, ws->TS_FFT, ws->FabX_Raw, FFTW_FORWARD, fft_plan_flags )) != NULL, XLAL_EFAILED, "fftwf_plan_dft_1d() failed\n");
           LAL_FFTW_WISDOM_UNLOCK;
           ws->numSamplesFFT = numSamplesFFT;
           ws->decimateFFT = decimateFFT;
@@ -342,9 +351,13 @@ XLALSetupFstatResamp ( void **method_data,
         XLAL_CHECK ( (ws->SRCtimes_DET->data = XLALRealloc ( ws->SRCtimes_DET->data, numSamplesMax_SRC * sizeof(REAL8) )) != NULL, XLAL_ENOMEM );
         ws->SRCtimes_DET->length = numSamplesMax_SRC;
       }
+
     } // end: if shared workspace given
   else
     {
+      int fft_plan_flags=FFTW_MEASURE;
+      double fft_plan_timeout= FFTW_NO_TIMELIMIT ;
+
       XLAL_CHECK ( (ws = XLALCalloc ( 1, sizeof(*ws))) != NULL, XLAL_ENOMEM );
       XLAL_CHECK ( (ws->TStmp1_SRC   = XLALCreateCOMPLEX8Vector ( numSamplesMax_SRC )) != NULL, XLAL_EFUNC );
       XLAL_CHECK ( (ws->TStmp2_SRC   = XLALCreateCOMPLEX8Vector ( numSamplesMax_SRC )) != NULL, XLAL_EFUNC );
@@ -354,7 +367,9 @@ XLALSetupFstatResamp ( void **method_data,
       XLAL_CHECK ( (ws->TS_FFT   = fftw_malloc ( numSamplesFFT * sizeof(COMPLEX8) )) != NULL, XLAL_ENOMEM );
 
       LAL_FFTW_WISDOM_LOCK;
-      XLAL_CHECK ( (ws->fftplan = fftwf_plan_dft_1d ( numSamplesFFT, ws->TS_FFT, ws->FabX_Raw, FFTW_FORWARD, FFTW_MEASURE )) != NULL, XLAL_EFAILED, "fftwf_plan_dft_1d() failed\n");
+      XLALGetFFTPlanHints (& fft_plan_flags , & fft_plan_timeout);
+      fftw_set_timelimit( fft_plan_timeout );
+      XLAL_CHECK ( (ws->fftplan = fftwf_plan_dft_1d ( numSamplesFFT, ws->TS_FFT, ws->FabX_Raw, FFTW_FORWARD, fft_plan_flags )) != NULL, XLAL_EFAILED, "fftwf_plan_dft_1d() failed\n");
       LAL_FFTW_WISDOM_UNLOCK;
       ws->numSamplesFFT = numSamplesFFT;
       ws->decimateFFT = decimateFFT;
@@ -362,12 +377,19 @@ XLALSetupFstatResamp ( void **method_data,
       common->workspace = ws;
     } // end: if we create our own workspace
 
-#if COLLECT_TIMING
-  // Set up timing log file
-  resamp->timingLogFile = optArgs->timingLogFile;
-#else
-  resamp->timingLogFile = NULL;
-#endif
+  // initialize struct for collecting timing data, store invariant 'meta' quantities about this setup
+  XLAL_INIT_MEM ( resamp->timingInfo );
+  resamp->timingInfo.collectTiming = optArgs->collectTiming;	// whether or not to collect timing info
+  resamp->timingInfo.Resolution = TspanXMax / TspanFFT;
+  resamp->timingInfo.NsFFT0 = numSamplesFFT0;
+  resamp->timingInfo.NsFFT  = numSamplesFFT;
+
+  resamp->timingInfo.Ndet = numDetectors;
+  UINT4 numSFTs = 0;
+  for ( UINT4 X = 0; X < numDetectors; X ++ ) {
+    numSFTs += common->multiDetectorStates->data[X]->length;
+  }
+  resamp->timingInfo.Nsft = numSFTs;
 
   return XLAL_SUCCESS;
 
@@ -392,20 +414,24 @@ XLALComputeFstatResamp ( FstatResults* Fstats,
 
   ResampWorkspace *ws = (ResampWorkspace*) common->workspace;
 
-#if COLLECT_TIMING
-  // collect internal timing info
-  XLAL_INIT_MEM ( ws->timingInfo );
-  ResampTimingInfo *ti = &(ws->timingInfo);
-  REAL8 ticStart,tocEnd;
-  ticStart = XLALGetCPUTime();
-  REAL8 tic,toc;
-#endif
-
   // ----- handy shortcuts ----------
   PulsarDopplerParams thisPoint = Fstats->doppler;
   const MultiCOMPLEX8TimeSeries *multiTimeSeries_DET = resamp->multiTimeSeries_DET;
   UINT4 numDetectors = multiTimeSeries_DET->length;
 
+  // collect internal timing info
+  ResampTimingInfo *ti = &(resamp->timingInfo);
+  REAL8 ticStart = 0, tocEnd = 0;
+  REAL8 tic = 0, toc = 0;
+  if ( ti->collectTiming ) {
+    XLAL_INIT_MEM ( ti->Tau );	// re-set all timings to 0 at beginning of each Fstat-call
+    ti->NFbin = Fstats->numFreqBins;
+
+    ticStart = XLALGetCPUTime();
+  }
+  // store pointer to timing-info storage in workspace (for use by XLALComputeFaFb_Resamp() )
+  ws->timingInfo = ti;
+
   // ============================== BEGIN: handle buffering =============================
   BOOLEAN same_skypos = (resamp->prev_doppler.Alpha == thisPoint.Alpha) && (resamp->prev_doppler.Delta == thisPoint.Delta);
   BOOLEAN same_refTime = ( GPSDIFF ( resamp->prev_doppler.refTime, thisPoint.refTime ) == 0 );
@@ -417,17 +443,21 @@ XLALComputeFstatResamp ( FstatResults* Fstats,
     (resamp->prev_doppler.argp == thisPoint.argp);
 
   // ----- not same skypos+binary+refTime? --> re-compute SRC-frame timeseries, AM-coeffs and store in buffer
-#if COLLECT_TIMING
-  tic = XLALGetCPUTime();
-#endif
+  if ( ti->collectTiming ) {
+    tic = XLALGetCPUTime();
+  }
+
   if ( ! ( same_skypos && same_refTime && same_binary) )
     {
       XLAL_CHECK ( XLALBarycentricResampleMultiCOMPLEX8TimeSeries ( resamp, &thisPoint, common ) == XLAL_SUCCESS, XLAL_EFUNC );
+      // record barycenter parameters in order to allow re-usal of this result ('buffering')
+      resamp->prev_doppler = thisPoint;
     }
-#if COLLECT_TIMING
-  toc = XLALGetCPUTime();
-  ti->tauBary = (toc-tic);
-#endif
+
+  if ( ti->collectTiming ) {
+    toc = XLALGetCPUTime();
+    ti->Tau.Bary = (toc-tic);
+  }
   MultiCOMPLEX8TimeSeries *multiTimeSeries_SRC_a = resamp->multiTimeSeries_SRC_a;
   MultiCOMPLEX8TimeSeries *multiTimeSeries_SRC_b = resamp->multiTimeSeries_SRC_b;
 
@@ -436,9 +466,9 @@ XLALComputeFstatResamp ( FstatResults* Fstats,
   // ----- workspace that depends on number of output frequency bins 'numFreqBins' ----------
   UINT4 numFreqBins = Fstats->numFreqBins;
 
-#if COLLECT_TIMING
-  tic = XLALGetCPUTime();
-#endif
+  if ( ti->collectTiming ) {
+    tic = XLALGetCPUTime();
+  }
 
   // NOTE: we try to use as much existing memory as possible in FstatResults, so we only
   // allocate local 'workspace' storage in case there's not already a vector allocated in FstatResults for it
@@ -478,12 +508,12 @@ XLALComputeFstatResamp ( FstatResults* Fstats,
     ws->numFreqBinsAlloc = numFreqBins;	// keep track of allocated array length
   }
   ws->numFreqBinsOut = numFreqBins;
-  // ====================================================================================================
 
-#if COLLECT_TIMING
-  toc = XLALGetCPUTime();
-  ti->tauMem = (toc-tic);	// this one doesn't scale with number of detector!
-#endif
+  if ( ti->collectTiming ) {
+    toc = XLALGetCPUTime();
+    ti->Tau.Mem = (toc-tic);	// this one doesn't scale with number of detector!
+  }
+  // ====================================================================================================
 
   // loop over detectors
   for ( UINT4 X=0; X < numDetectors; X++ )
@@ -500,9 +530,9 @@ XLALComputeFstatResamp ( FstatResults* Fstats,
       // compute {Fa^X(f_k), Fb^X(f_k)}: results returned via workspace ws
       XLAL_CHECK ( XLALComputeFaFb_Resamp ( ws, thisPoint, common->dFreq, TimeSeriesX_SRC_a, TimeSeriesX_SRC_b ) == XLAL_SUCCESS, XLAL_EFUNC );
 
-#if COLLECT_TIMING
-      tic = XLALGetCPUTime();
-#endif
+      if ( ti->collectTiming ) {
+        tic = XLALGetCPUTime();
+      }
       if ( X == 0 )
         { // avoid having to memset this array: for the first detector we *copy* results
           for ( UINT4 k = 0; k < numFreqBins; k++ )
@@ -519,11 +549,13 @@ XLALComputeFstatResamp ( FstatResults* Fstats,
               ws->Fb_k[k] += ws->FbX_k[k];
             }
         } // end:if X>0
-#if COLLECT_TIMING
-      toc = XLALGetCPUTime();
-      ti->tauSumFabX += (toc-tic);
-      tic = toc;
-#endif
+
+      if ( ti->collectTiming ) {
+        toc = XLALGetCPUTime();
+        ti->Tau.SumFabX += (toc-tic);
+        tic = toc;
+      }
+
       // ----- if requested: compute per-detector Fstat_X_k
       if ( whatToCompute & FSTATQ_2F_PER_DET )
         {
@@ -537,18 +569,20 @@ XLALComputeFstatResamp ( FstatResults* Fstats,
               Fstats->twoFPerDet[X][k] = XLALComputeFstatFromFaFb ( ws->FaX_k[k], ws->FbX_k[k], AdX, BdX, CdX, EdX, DdX_inv );
             }  // for k < numFreqBins
         } // end: if compute F_X
-#if COLLECT_TIMING
-      toc = XLALGetCPUTime();
-      ti->tauFab2F += ( toc - tic );
-#endif
+
+      if ( ti->collectTiming ) {
+        toc = XLALGetCPUTime();
+        ti->Tau.Fab2F += ( toc - tic );
+      }
 
     } // for X < numDetectors
 
-#if COLLECT_TIMING
-  ti->tauSumFabX /= numDetectors;
-  ti->tauFab2F /= numDetectors;
-  tic = XLALGetCPUTime();
-#endif
+  if ( ti->collectTiming ) {
+    ti->Tau.SumFabX /= numDetectors;
+    ti->Tau.Fab2F /= numDetectors;
+    tic = XLALGetCPUTime();
+  }
+
   if ( whatToCompute & FSTATQ_2F )
     {
       const REAL4 Ad = resamp->Mmunu.Ad;
@@ -561,10 +595,11 @@ XLALComputeFstatResamp ( FstatResults* Fstats,
           Fstats->twoF[k] = XLALComputeFstatFromFaFb ( ws->Fa_k[k], ws->Fb_k[k], Ad, Bd, Cd, Ed, Dd_inv );
         }
     } // if FSTATQ_2F
-#if COLLECT_TIMING
+
+  if ( ti->collectTiming ) {
       toc = XLALGetCPUTime();
-      ti->tauFab2F += ( toc - tic );
-#endif
+      ti->Tau.Fab2F += ( toc - tic );
+  }
 
   // Return F-atoms per detector
   if (whatToCompute & FSTATQ_ATOMS_PER_DET) {
@@ -595,30 +630,26 @@ XLALComputeFstatResamp ( FstatResults* Fstats,
       ws->FbX_k = NULL;
     }
 
-#if COLLECT_TIMING
-  // timings are per-detector
-  tocEnd = XLALGetCPUTime();
-  ti->tauTotal = (tocEnd - ticStart);
-  // rescale all relevant timings to single-IFO case
-  ti->tauTotal /= numDetectors;
-  ti->tauBary  /= numDetectors;
-  ti->tauSpin  /= numDetectors;
-  ti->tauFFT   /= numDetectors;
-  ti->tauNorm  /= numDetectors;
+  if ( ti->collectTiming ) {
+    // timings are per-detector
+    tocEnd = XLALGetCPUTime();
+    Timings_t *Tau = &(ti->Tau);
+    Tau->Total = (tocEnd - ticStart);
+    // rescale all relevant timings to single-IFO case
+    Tau->Total /= numDetectors;
+    Tau->Bary  /= numDetectors;
+    Tau->Spin  /= numDetectors;
+    Tau->FFT   /= numDetectors;
+    Tau->Norm  /= numDetectors;
+    Tau->Copy  /= numDetectors;
 
-  // compute 'fundamental' timing numbers per template per detector
-  ti->tauF1NoBuf = ti->tauTotal / numFreqBins;
-  ti->tauF1Buf   = (ti->tauTotal - ti->tauBary - ti->tauMem) / numFreqBins;
-
-  // pass out timings to test/benchmark programs
-  Fstat_tauF1NoBuf = ti->tauF1NoBuf;
-  Fstat_tauF1Buf = ti->tauF1Buf;
-
-  // write timing info to log file
-  if ( resamp->timingLogFile != NULL ) {
-    AppendResampInfo2File ( resamp->timingLogFile, common, resamp );
+    // compute 'fundamental' per output bin timing numbers and timing-model coefficients
+    ti->tau_RS	= (Tau->Total - Tau->Bary) / ti->NFbin;
+    ti->tau_FFT	= Tau->FFT / ti->NsFFT;
+    ti->tau_spin= Tau->Spin / (ti->Resolution * ti->NsFFT );
+    ti->tau_Fbin= (Tau->Copy + Tau->Norm + Tau->SumFabX + Tau->Fab2F) / ti->NFbin;
+    ti->tau_bary= Tau->Bary / (ti->Resolution * ti->NsFFT);
   }
-#endif
 
   return XLAL_SUCCESS;
 
@@ -650,62 +681,73 @@ XLALComputeFaFb_Resamp ( ResampWorkspace *restrict ws,				//!< [in,out] pre-allo
   UINT4 maxOutputBin = offset_bins + (ws->numFreqBinsOut-1) * ws->decimateFFT;
   XLAL_CHECK ( maxOutputBin < ws->numSamplesFFT, XLAL_EDOM, "Highest output frequency bin outside available band: [maxOutputBin = %d] >= [numSamplesFFT = %d]\n", maxOutputBin, ws->numSamplesFFT );
 
-#if COLLECT_TIMING
-  // collect some internal timing info
-  ResampTimingInfo *ti = &(ws->timingInfo);
-  REAL8 tic,toc;
-  tic = XLALGetCPUTime();
-#endif
+  ResampTimingInfo *ti = ws->timingInfo;
+  REAL8 tic = 0, toc = 0;
 
   XLAL_CHECK ( ws->numSamplesFFT >= TimeSeries_SRC_a->data->length, XLAL_EFAILED, "[numSamplesFFT = %d] < [len(TimeSeries_SRC_a) = %d]\n", ws->numSamplesFFT, TimeSeries_SRC_a->data->length );
   XLAL_CHECK ( ws->numSamplesFFT >= TimeSeries_SRC_b->data->length, XLAL_EFAILED, "[numSamplesFFT = %d] < [len(TimeSeries_SRC_b) = %d]\n", ws->numSamplesFFT, TimeSeries_SRC_b->data->length );
 
+  if ( ti->collectTiming ) {
+    tic = XLALGetCPUTime();
+  }
   memset ( ws->TS_FFT, 0, ws->numSamplesFFT * sizeof(ws->TS_FFT[0]) );
   // ----- compute FaX_k
   // apply spindown phase-factors, store result in zero-padded timeseries for 'FFT'ing
   XLAL_CHECK ( XLALApplySpindownAndFreqShift ( ws->TS_FFT, TimeSeries_SRC_a, &thisPoint, freqShift ) == XLAL_SUCCESS, XLAL_EFUNC );
 
-#if COLLECT_TIMING
-  toc = XLALGetCPUTime();
-  ti->tauSpin += ( toc - tic);
-  tic = toc;
-#endif
+  if ( ti->collectTiming ) {
+    toc = XLALGetCPUTime();
+    ti->Tau.Spin += ( toc - tic);
+    tic = toc;
+  }
 
   // Fourier transform the resampled Fa(t)
   fftwf_execute ( ws->fftplan );
 
+  if ( ti->collectTiming ) {
+    toc = XLALGetCPUTime();
+    ti->Tau.FFT += ( toc - tic);
+    tic = toc;
+  }
+
   for ( UINT4 k = 0; k < ws->numFreqBinsOut; k++ ) {
     ws->FaX_k[k] = ws->FabX_Raw [ offset_bins + k * ws->decimateFFT ];
   }
 
-#if COLLECT_TIMING
-  toc = XLALGetCPUTime();
-  ti->tauFFT += ( toc - tic);
-  tic = toc;
-#endif
+  if ( ti->collectTiming ) {
+    toc = XLALGetCPUTime();
+    ti->Tau.Copy += ( toc - tic);
+    tic = toc;
+  }
 
   // ----- compute FbX_k
   // apply spindown phase-factors, store result in zero-padded timeseries for 'FFT'ing
   XLAL_CHECK ( XLALApplySpindownAndFreqShift ( ws->TS_FFT, TimeSeries_SRC_b, &thisPoint, freqShift ) == XLAL_SUCCESS, XLAL_EFUNC );
 
-#if COLLECT_TIMING
-  toc = XLALGetCPUTime();
-  ti->tauSpin += ( toc - tic);
-  tic = toc;
-#endif
+  if ( ti->collectTiming ) {
+    toc = XLALGetCPUTime();
+    ti->Tau.Spin += ( toc - tic);
+    tic = toc;
+  }
 
   // Fourier transform the resampled Fa(t)
   fftwf_execute ( ws->fftplan );
 
+  if ( ti->collectTiming ) {
+    toc = XLALGetCPUTime();
+    ti->Tau.FFT += ( toc - tic);
+    tic = toc;
+  }
+
   for ( UINT4 k = 0; k < ws->numFreqBinsOut; k++ ) {
     ws->FbX_k[k] = ws->FabX_Raw [ offset_bins + k * ws->decimateFFT ];
   }
 
-#if COLLECT_TIMING
-  toc = XLALGetCPUTime();
-  ti->tauFFT += ( toc - tic);
-  tic = toc;
-#endif
+  if ( ti->collectTiming ) {
+    toc = XLALGetCPUTime();
+    ti->Tau.Copy += ( toc - tic);
+    tic = toc;
+  }
 
   // ----- normalization factors to be applied to Fa and Fb:
   const REAL8 dtauX = GPSDIFF ( TimeSeries_SRC_a->epoch, thisPoint.refTime );
@@ -720,11 +762,11 @@ XLALComputeFaFb_Resamp ( ResampWorkspace *restrict ws,				//!< [in,out] pre-allo
       ws->FbX_k[k] *= normX_k;
     } // for k < numFreqBinsOut
 
-#if COLLECT_TIMING
-  toc = XLALGetCPUTime();
-  ti->tauNorm += ( toc - tic);
-  tic = toc;
-#endif
+  if ( ti->collectTiming ) {
+    toc = XLALGetCPUTime();
+    ti->Tau.Norm += ( toc - tic);
+    tic = toc;
+  }
 
   return XLAL_SUCCESS;
 
@@ -882,6 +924,9 @@ XLALBarycentricResampleMultiCOMPLEX8TimeSeries ( ResampMethodData *resamp,		// [
       // make sure detector-frame timesteps to interpolate to are initialized to 0, in case of gaps
       memset ( ws->SRCtimes_DET->data, 0, ws->SRCtimes_DET->length * sizeof(ws->SRCtimes_DET->data[0]) );
 
+      memset ( ws->TStmp1_SRC->data, 0, ws->TStmp1_SRC->length * sizeof(ws->TStmp1_SRC->data[0]) );
+      memset ( ws->TStmp2_SRC->data, 0, ws->TStmp2_SRC->length * sizeof(ws->TStmp2_SRC->data[0]) );
+
       REAL8 tStart_DET_0 = GPSGETREAL8 ( &(Timestamps_DETX->data[0]) );// START time of the SFT at the detector
 
       // loop over SFT timestamps and compute the detector frame time samples corresponding to uniformly sampled SRC time samples
@@ -956,3 +1001,95 @@ XLALBarycentricResampleMultiCOMPLEX8TimeSeries ( ResampMethodData *resamp,		// [
   return XLAL_SUCCESS;
 
 } // XLALBarycentricResampleMultiCOMPLEX8TimeSeries()
+
+// export timing constants from internally-stored 'method data' struct
+int
+XLALGetFstatTiming_Resamp ( const void* method_data, REAL8 *tauF1Buf, REAL8 *tauF1NoBuf )
+{
+  XLAL_CHECK ( method_data != NULL, XLAL_EINVAL );
+  XLAL_CHECK ( (tauF1Buf != NULL) && (tauF1NoBuf != NULL), XLAL_EINVAL );
+
+  const ResampMethodData *resamp = (const ResampMethodData *)method_data;
+
+  // time per template assuming perfect buffering
+  (*tauF1Buf) = resamp->timingInfo.tau_RS;
+  // 'raw' time per freq-bin, potentially including barycentering (if no buffering)
+  (*tauF1NoBuf) = resamp->timingInfo.Tau.Total / resamp->timingInfo.NFbin;
+
+  return XLAL_SUCCESS;
+
+} // XLALGetFstatTiming_Resamp()
+
+// append detailed (method-specific) timing info to given file
+int
+AppendFstatTimingInfo2File_Resamp ( const void* method_data, FILE *fp, BOOLEAN printHeader )
+{
+  XLAL_CHECK ( method_data != NULL, XLAL_EINVAL );
+  XLAL_CHECK ( fp != NULL, XLAL_EINVAL );
+
+  const ResampMethodData *resamp = (const ResampMethodData *)method_data;
+  // print header if requested
+  if ( printHeader ) {
+    fprintf (fp, "%%%% ----- Resampling F-stat timing: -----\n");
+    fprintf (fp, "%%%% Measured time (in seconds) per F-stat frequency bin per detector (excluding barycentering):\n");
+    fprintf (fp, "%%%% tau_RS = (TauTotal - TauBary) / NFbin\n");
+    fprintf (fp, "%%%% tau_RS-predicted = tau_Fbin + (NsFFT/NFbin) * ( R * tau_spin + tau_FFT )\n");
+    fprintf (fp, "%%%% with the frequency resolution in natural units, R = Tspan / T_FFT = NsSRC / NsFFT,\n");
+    fprintf (fp, "%%%% Total time per detector generally contains an additional barycentering contribution:\n");
+    fprintf (fp, "%%%% TauTotal = NFbin * tauRS + b * R * NsFFT * tau_bary\n");
+    fprintf (fp, "%%%% where the buffering weight b = 1/N_{f1dot,f2dot,..} goes to 0 for many spindowns per sky+binary template\n");
+
+    fprintf (fp, "%%%%%8s %8s %8s %6s %6s", "NFbin", "NsFFT0", "l2NsFFT", "Ndet", "R" );
+    fprintf (fp, " %10s %10s %10s %10s %10s %10s", "TauTotal", "tau_RS", "tau_Fbin", "tau_FFT", "tau_spin", "tau_bary" );
+    fprintf (fp, "\n");
+  }
+
+  const ResampTimingInfo *ti = &(resamp->timingInfo);
+  fprintf (fp, "%10d %8d %8.2f %6d %6.3f",
+           ti->NFbin, ti->NsFFT0, log2(ti->NsFFT), ti->Ndet, ti->Resolution );
+
+  fprintf (fp, " %10.1e %10.1e %10.1e %10.1e %10.1e %10.1e",
+           ti->Tau.Total, ti->tau_RS, ti->tau_Fbin, ti->tau_FFT, ti->tau_spin, ti->tau_bary );
+
+  fprintf (fp, "\n");
+
+  return XLAL_SUCCESS;
+} // AppendFstatTimingInfo2File_Resamp()
+
+
+
+static void
+XLALGetFFTPlanHints ( int * planMode,
+                      double * planGenTimeoutSeconds
+                      )
+{
+  char * planMode_env = getenv("LAL_FSTAT_FFT_PLAN_MODE");
+  char * planGenTimeout_env = getenv("LAL_FSTAT_FFT_PLAN_TIMEOUT");;
+  int fft_plan_flags=FFTW_MEASURE;
+  double fft_plan_timeout= FFTW_NO_TIMELIMIT ;
+
+  if ( planGenTimeout_env ) {
+    char * end;
+    fft_plan_timeout=strtod(planGenTimeout_env,& end);
+    if(end[0] != '\0') {
+      fft_plan_timeout=FFTW_NO_TIMELIMIT;
+    }
+  }
+
+  if ( planMode_env ) {
+    if ( strcmp(planMode_env , "ESTIMATE" ) == 0 ) {
+      fft_plan_flags=FFTW_ESTIMATE;
+    }
+
+    if ( strcmp(planMode_env , "MEASURE" ) == 0 ) {
+      fft_plan_flags=FFTW_MEASURE;
+    }
+
+    if ( strcmp(planMode_env , "PATIENT" ) == 0 ) {
+      fft_plan_flags=FFTW_PATIENT;
+    }
+  }
+  *planMode=fft_plan_flags;
+  *planGenTimeoutSeconds=fft_plan_timeout;
+} // XLALGetFFTPlanHints
+
