@@ -133,7 +133,13 @@ static UINT8 PulsarHash( const void *elem );
 static UINT8 PulsarHash( const void *elem )
 {
   if( !elem ) { XLAL_ERROR(XLAL_EINVAL); }
-  size_t len = strnlen(((const hash_elem *)elem)->name, PULSAR_PARNAME_MAX );
+  // size_t len = strnlen(((const hash_elem *)elem)->name, PULSAR_PARNAME_MAX );
+  // use this instead of strnlen
+  const char *s = ((const hash_elem *)elem)->name;
+  size_t len;
+  for (len = 0; len < PULSAR_PARNAME_MAX; len++, s++) {
+    if (!*s) { break; }
+  }
   return( XLALCityHash64(((const hash_elem *)elem)->name, len) );
 }
 
